'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var _regeneratorRuntime = _interopDefault(require('regenerator-runtime'));
var unistUtilVisit = require('unist-util-visit');
var jsdom = require('jsdom');
var rangeParser = _interopDefault(require('parse-numeric-range'));
var shiki = _interopDefault(require('shiki'));
var sanitizeHtml = _interopDefault(require('sanitize-html'));

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) {
  try {
    var info = gen[key](arg);
    var value = info.value;
  } catch (error) {
    reject(error);
    return;
  }

  if (info.done) {
    resolve(value);
  } else {
    Promise.resolve(value).then(_next, _throw);
  }
}

function _asyncToGenerator(fn) {
  return function () {
    var self = this,
        args = arguments;
    return new Promise(function (resolve, reject) {
      var gen = fn.apply(self, args);

      function _next(value) {
        asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value);
      }

      function _throw(err) {
        asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err);
      }

      _next(undefined);
    });
  };
}

function createRemarkPlugin(options) {
  if (options === void 0) {
    options = {};
  }

  return function () {
    return /*#__PURE__*/function () {
      var _ref = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee(tree) {
        var _options, _options$sanitizeOpti, sanitizeOptions, _options$shikiOptions, shikiOptions, _options$tokensMap, tokensMap, _options$onVisitLine, onVisitLine, _options$onVisitHighl, onVisitHighlightedLine, _options$onVisitHighl2, onVisitHighlightedWord, _options$ignoreUnknow, ignoreUnknownLanguage, highlighter, loadedLanguages, inlineCode, blockCode;

        return _regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                blockCode = function _blockCode(node) {
                  var _node$meta$match$, _node$meta$match, _node$meta, _node$meta$match2, _node$meta$match$2, _node$meta$match3;

                  var lang = ignoreUnknownLanguage && !loadedLanguages.includes(node.lang) ? null : node.lang;
                  node.type = 'html';
                  var highlighted = highlighter.codeToHtml(node.value, lang);
                  var lineNumbers = node.meta ? rangeParser((_node$meta$match$ = (_node$meta$match = node.meta.match(/{(.*)}/)) == null ? void 0 : _node$meta$match[1]) != null ? _node$meta$match$ : '') : [];
                  var word = (_node$meta = node.meta) == null ? void 0 : (_node$meta$match2 = _node$meta.match(/\/(.*)\//)) == null ? void 0 : _node$meta$match2[1];
                  var wordNumbers = node.meta ? rangeParser((_node$meta$match$2 = (_node$meta$match3 = node.meta.match(/\/.*\/([^\s]*)/)) == null ? void 0 : _node$meta$match3[1]) != null ? _node$meta$match$2 : '') : [];
                  var visitedWordsCount = 0;
                  var dom = new jsdom.JSDOM(highlighted);
                  dom.window.document.body.querySelectorAll('.line').forEach(function (node, i) {
                    onVisitLine(node);

                    if (lineNumbers.includes(i + 1)) {
                      onVisitHighlightedLine(node);
                    }

                    if (word && node.innerHTML.includes(word)) {
                      visitedWordsCount++;

                      if (wordNumbers.length === 0 || wordNumbers.includes(visitedWordsCount)) {
                        var splitByBreakChar = new RegExp("\\b" + word + "\\b");
                        var adjacentText = node.innerHTML.split(splitByBreakChar);
                        node.innerHTML = adjacentText.map(function (txt, i) {
                          return "" + txt + (i !== adjacentText.length - 1 ? "<span data-mdx-pretty-code-word>" + word + "</span>" : '');
                        }).flat().join('');
                        node.querySelectorAll('[data-mdx-pretty-code-word]').forEach(function (wordNode) {
                          wordNode.removeAttribute('data-mdx-pretty-code-word');
                          onVisitHighlightedWord(wordNode);
                        });
                      }
                    }
                  });
                  node.value = sanitizeHtml(dom.window.document.body.innerHTML, sanitizeOptions);
                };

                inlineCode = function _inlineCode(node) {
                  var _node$value$match;

                  var meta = (_node$value$match = node.value.match(/{:([a-zA-Z.-]+)}$/)) == null ? void 0 : _node$value$match[1];

                  if (!meta) {
                    return;
                  }

                  node.type = 'html'; // It's a token, not a lang

                  if (meta[0] === '.') {
                    var _shikiOptions$theme$t, _shikiOptions$theme$t2;

                    if (typeof shikiOptions.theme === 'string') {
                      throw new Error('MDX Pretty Code: Must be using a JSON theme object to use tokens.');
                    }

                    var color = (_shikiOptions$theme$t = (_shikiOptions$theme$t2 = shikiOptions.theme.tokenColors.find(function (_ref2) {
                      var _tokensMap$meta$slice;

                      var scope = _ref2.scope;
                      return scope == null ? void 0 : scope.includes((_tokensMap$meta$slice = tokensMap[meta.slice(1)]) != null ? _tokensMap$meta$slice : meta.slice(1));
                    })) == null ? void 0 : _shikiOptions$theme$t2.settings.foreground) != null ? _shikiOptions$theme$t : 'inherit';
                    node.value = sanitizeHtml("<span data-mdx-pretty-code data-color=\"" + color + "\"><span>" + node.value.replace(/{:[a-zA-Z.-]+}/, '') + "</span></span>", sanitizeOptions);
                    return;
                  }

                  var highlighted = highlighter.codeToHtml(node.value.replace(/{:\w+}/, ''), meta);
                  var dom = new jsdom.JSDOM(highlighted);
                  var pre = dom.window.document.querySelector('pre');
                  node.value = sanitizeHtml("<span data-mdx-pretty-code>" + pre.innerHTML + "</span>", sanitizeOptions);
                };

                _options = options, _options$sanitizeOpti = _options.sanitizeOptions, sanitizeOptions = _options$sanitizeOpti === void 0 ? {
                  allowedAttributes: {
                    code: ['style'],
                    span: ['data-color', 'data-mdx-pretty-code', 'style', 'class']
                  }
                } : _options$sanitizeOpti, _options$shikiOptions = _options.shikiOptions, shikiOptions = _options$shikiOptions === void 0 ? {} : _options$shikiOptions, _options$tokensMap = _options.tokensMap, tokensMap = _options$tokensMap === void 0 ? {} : _options$tokensMap, _options$onVisitLine = _options.onVisitLine, onVisitLine = _options$onVisitLine === void 0 ? function () {} : _options$onVisitLine, _options$onVisitHighl = _options.onVisitHighlightedLine, onVisitHighlightedLine = _options$onVisitHighl === void 0 ? function () {} : _options$onVisitHighl, _options$onVisitHighl2 = _options.onVisitHighlightedWord, onVisitHighlightedWord = _options$onVisitHighl2 === void 0 ? function () {} : _options$onVisitHighl2, _options$ignoreUnknow = _options.ignoreUnknownLanguage, ignoreUnknownLanguage = _options$ignoreUnknow === void 0 ? true : _options$ignoreUnknow;
                _context.next = 5;
                return shiki.getHighlighter(shikiOptions);

              case 5:
                highlighter = _context.sent;
                loadedLanguages = highlighter.getLoadedLanguages();
                unistUtilVisit.visit(tree, 'inlineCode', inlineCode);
                unistUtilVisit.visit(tree, 'code', blockCode);

              case 9:
              case "end":
                return _context.stop();
            }
          }
        }, _callee);
      }));

      return function (_x) {
        return _ref.apply(this, arguments);
      };
    }();
  };
}

exports.createRemarkPlugin = createRemarkPlugin;
//# sourceMappingURL=syntax-highlighter.cjs.development.js.map
